/* vi:set cindent tabstop=2 shiftwidth=2: */
/*
 * libvxfs - library for reading Veritas Journaled FileSystem (VxFS)
 * Copyright (c) 1999 Martin Hinner <mhi@penguin.cz>
 *
 * block.c: VxFS block routines and UnixWare vtoc support
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: block.c,v 0.3 1999/12/14 19:04:35 mhi Exp root $
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>

#include <libvxfs.h>

int
vxopendev (struct vxfs *fs, char *device)
{
	fs->bsize = 1024;

	fs->fd = open (device, O_RDONLY);
	if (fs->fd < 0)
		{
			perror (device);
			return -1;
		}
	return 0;
}

void
vxclosedev (struct vxfs *fs)
{
	if (fs->fd >= 0)
		{
			close (fs->fd);
			fs->fd = -1;
			fs->bsize = 0;
		}
}

int
vxsetpart (struct vxfs *fs, int part)
{
	struct uw_vtoc *vtoc;

	if (part >= V_NUMPAR)
		{
			if (vxverbose)
				fprintf (stderr, "vxsetpart: Invalid partition %u\n", part);
			return -1;
		}

	vtoc = vxreaduwvtoc (fs);
	if (vtoc == NULL)
		{

			/* No partition specified, silently ignore */
			if (part == 0)
				return 0;

			if (vxverbose)
				fprintf (stderr, "vxsetpart: vxreaduwvtoc returned NULL\n");
			return -1;
		}

	if (vtoc->part[part].tag == V_UNUSED)
		{
			if (vxverbose)
				fprintf (stderr, "vxsetpart: Attemp to mount unused partition\n");
			return -1;
		}

	fs->partoffset = vtoc->part[part].start;

	free (vtoc);
	return 0;
}


struct uw_vtoc *
vxreaduwvtoc (struct vxfs *fs)
{
	struct uw_vtoc *vtoc;
	struct uw_disklabel *disklabel;
	struct uw_dospt *dospt;
	int i, npart;

	dospt = (struct uw_dospt *) malloc (VTOC_BSIZE);
	if (!dospt)
		{
			if (vxverbose)
				fprintf (stderr, "vxreaduwvtoc: Not enough memory!\n");
			return 0;
		}

	if (lseek (fs->fd, 0, SEEK_SET) < 0)
		{
			if (vxverbose)
				perror ("vxreaduwvtoc: lseek");
			return 0;
		}

	if (read (fs->fd, dospt, VTOC_BSIZE) != VTOC_BSIZE)
		{
			if (vxverbose)
				perror ("vxreaduwvtoc: read");
			return 0;
		}

	npart = -1;
	for (i = 0; i < 4; i++)
		{
			if (dospt->part[i].sys_ind == VTOC_DOSTYPE)
				npart = i;
		}

	if (npart == -1)
		{
			if (vxverbose)
				fprintf (stderr, "vxreaduwvtoc: No UnixWare partition found!\n");
			free (dospt);
			return 0;
		}

	disklabel = (struct uw_disklabel *) malloc (VTOC_BSIZE);
	if (!disklabel)
		{
			if (vxverbose)
				fprintf (stderr, "vxreaduwvtoc: Not enough memory!\n");
			return 0;
		}

	if (lseek (fs->fd, VTOC_BSIZE *
						 (dospt->part[npart].start_sect + VTOC_SECTOR), SEEK_SET) < 0)
		{
			if (vxverbose)
				perror ("vxreaduwvtoc: lseek");
			return 0;
		}

	if (read (fs->fd, disklabel, VTOC_BSIZE) != VTOC_BSIZE)
		{
			if (vxverbose)
				perror ("vxreaduwvtoc: read");
			return 0;
		}

	if (disklabel->vtoc.sanity != VTOC_SANE)
		{
			free (disklabel);
			return 0;
		}

	vtoc = (struct uw_vtoc *) malloc (sizeof (struct uw_vtoc));
	if (vtoc == NULL)
		{
			if (vxverbose)
				fprintf (stderr, "vxreaduwvtoc: Not enough memory!\n");
			return 0;
		}

	memcpy (vtoc, &disklabel->vtoc, sizeof (struct uw_vtoc));

	free (disklabel);

	return vtoc;
}

void *
vxread (struct vxfs *fs, int blknum, int count)
{
	void *blk;

	blk = malloc (fs->bsize * count);
	if (!blk)
		{
			if (vxverbose)
				fprintf (stderr, "vxread(%u): Not enough memory!\n", blknum);
			return 0;
		}

	if (lseek (fs->fd, (VTOC_BSIZE * fs->partoffset) +
						 (blknum * fs->bsize), SEEK_SET) < 0)
		{
			if (vxverbose)
				perror ("vxread: lseek");
			return 0;
		}
	if (read (fs->fd, blk, fs->bsize * count) != fs->bsize * count)
		{
			if (vxverbose)
				perror ("vxread: read");
			return 0;
		}
	return blk;
}
